/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#ifndef OMS_APP_H
#define OMS_APP_H

#include "io.h"
#include "basic_types.h"
#include "BinStatistic.h"

//--------------------------------------------------------------------------------------------------

class version
{
	const unsigned long v;
public:
	version(unsigned long vers ) : v( vers ) {}
	bool dne()                 { return v == NULL; } // null v means "does not exist"
	unsigned char major_BCD()  { return (unsigned char) (v >> 24); } // major version number in BCD
	void pr( const char* s)
	{
		int t = (v >> 12) & 0xF;
		tw.pr
		(
			"%s version %x.%d.%d%s%x\r", s, major_BCD(), (v >> 20) & 0xF, (v >> 16) & 0xF,
			t==2 ? "d" : t==4 ? "a" : t==6 ? "b" : t==8 ? "r" : "?", v & 0xFF
		);
	}
};

//--------------------------------------------------------------------------------------------------

class OMS_basic_app
{
protected:
	const long as;

public:
	 OMS_basic_app( long app_sig );
	~OMS_basic_app() { }
};

//--------------------------------------------------------------------------------------------------

#include "ctype.h" // for toupper()

template <class T> class OMS_node_info
{
	typedef UtilArray<T> TArray;
	typedef UtilPtr<TArray> TArrayPtr;
	static void strtoupper( const char *i, char *o ) { while ( (*o++ = toupper(*i++)) != 0 ) {} }
public:
	static err_ind select_by_name( const char*, TArrayPtr, UINT *id );
	static void print_nodes( TArrayPtr, const char *type );
};
	
//--------------------------------------------------------------------------------------------------

#include <stddef.h> // for typedef size_t for overloaded operator new
#include "data_structs.h"
#include "MIDI_lib.h"
#include "Rogus.h"

struct MyMidiIn : MIDIINCAPS {
	HMIDIIN handle;
	bool open;
	bool operator==(const MyMidiIn& m) { return (this == &m); }
};

struct MyMidiOut : MIDIOUTCAPS {
	HMIDIOUT handle;
	bool open;
	bool operator==(const MyMidiOut& m) { return (this == &m); }
};

struct MyQueueElement {
	DWORD dwParam1;
	DWORD dwParam2;
	UINT n;
};

class OMS_app : public OMS_basic_app
{
	typedef UtilArray<MyMidiIn> MyMidiInArray;
	typedef UtilPtr<MyMidiInArray> MyMidiInArrayPtr;
	typedef UtilArray<MyMidiOut> MyMidiOutArray;
	typedef UtilPtr<MyMidiOutArray> MyMidiOutArrayPtr;

	static DWORD our_start_time; // 3
	MyMidiInArrayPtr inlh;
	MyMidiOutArrayPtr onlh;
	static c_queue<MyQueueElement> cq;
	BinStatistic queueStat;

	static void CALLBACK my_read_hook(HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2);
	
	void init_timer();
	void* operator new(size_t); // 1
	
public:
	static parser *lp;

	 OMS_app( long app_sig  );
	~OMS_app();

	ioref open_input( const char* );
	ioref output_refnum( const char* );
	
	void closeAllInputs();
	void closeAllOutputs();
	void processQueue(); // called by scheduler

	int howManyOutputNodes(); // added this for PerChannel to use. 10/27/95 PP
	ioref getOutputRefnum(int); // added this for PerChannel to use 10/27/95 PP
	char* getOutputNodeName(ioref); // added this for Recorder to use 11/7/95 PP

	static long time() { return (timeGetTime()-our_start_time) * 2; } // ms to demims

	void write(const MIDI_msg *m);
};
/* 1. This private declaration of new() prevents dynamic allocation from taking place.  DO NOT 
try to change this code so that you can dynamically allocate an OMS_app. OMS_app uses 
Stroustrup's "resource acquisition is initialization" strategy (p. 308), which seems to only 
work for objects not allocated dynamically.
3. This point of this is to get around the wrapping problem if Windows is run for over 49 days.
By subtracting the start time of our program, we get around this problem, although we still will
have trouble if our program is run for over ~12 days, since ScheduleTime is signed (not unsigned)
and is in units of demimilliseconds, not milliseconds. */

#endif
// Authors: Ben Denckla, Patrick Pelletier
