/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#ifndef UTILPTR_H
#define UTILPTR_H

#include "basic_types.h"

template <class T>
class UtilPtr
{
	struct RefCounter
	{
		T* p;
		int count;
	};
	RefCounter* rc;
	
	void inc() { if (rc) rc->count++; }
	void dec() { if (rc && ((--(rc->count)) == 0)) { delete (rc->p); delete rc; rc = NULL; } }
	void setup(T* x) { rc = new RefCounter; rc->count = 1; rc->p = x; }
public:
	UtilPtr() { rc = NULL; }
	UtilPtr(T* x) { if (x != NULL) setup(x); else rc = NULL; }
	UtilPtr(const UtilPtr<T>& x) { rc = x.rc; inc(); }
	~UtilPtr() { dec(); }
	UtilPtr<T>& operator=(const UtilPtr<T>& x) { dec(); rc = x.rc; inc(); return *this; }
	UtilPtr<T>& operator=(T* x) { dec(); if (x != NULL) setup(x); else rc = NULL; return *this; }
	bool operator==(const UtilPtr<T>& x) const { return (rc == x.rc); }
	bool operator==(T* x) const { if (rc != NULL) return (rc->p == x); else return (x == NULL); }
	T* operator->() const { if (rc == NULL) return NULL; else return (rc->p); }
	T& operator*() const { return *operator->(); }
	operator T*() const { return operator->(); }
};

#endif
// Author: Patrick Pelletier
