/*
** Copyright 1995-1996 by the Massachusetts Institute of Technology.  All
** rights reserved. For more information, see the file COPYING which came
** with this source distribution.
*/

#ifndef SCORECREATIONINFO_H
#define SCORECREATIONINFO_H

#include "UtilPtr.h"
#include "UtilArray.h"
#include "Filedata.h"
#include "ScorePlayer.h"

typedef UtilPtr<Filedata> FiledataPtr;

class ScoreCreationInfo {
private:
	friend class Score;

	struct FuncEl {
		void (*func)(void*, ScorePlayer*);
		void* v;
		ScheduleTime time;
	};

	typedef UtilArray<bool> boolArray;
	typedef UtilArray<ioref> iorefArray;
	typedef UtilPtr<UtilString> UtilStringPtr;
	typedef UtilArray<UtilStringPtr> UtilStringPtrArray;

	FiledataPtr fd;
	uint2 division;
	boolArray track_included;
	iorefArray track_node;
	UtilStringPtrArray track_name;

	MIDI_msg** msgs;
	int num_msgs;
	int num_msgs_allocated;

	FuncEl** funcs;
	int num_funcs;
	int num_funcs_allocated;

	void calculate_track_names();
	void do_sort() { sort(msgs, num_msgs); sort(funcs, num_funcs); }

	// wish I could do these next two functions with a template, but can't figure out how
	static void sort (MIDI_msg** els, int len);
	static void sort (FuncEl** els, int len);

public:
	ScoreCreationInfo (FiledataPtr f);
	ScoreCreationInfo (uint2 div);
	~ScoreCreationInfo ();
	err_ind addTrack(const char* t, ioref n);
	int addRemainingTracks(ioref n);
	int addRemainingTracksAndLookupNodenames();
	void addMIDI_msg(const MIDI_msg& m);
	void addFunction(void (*func)(void*, ScorePlayer*), void* v, ScheduleTime t);
};

#endif
// Author: Patrick Pelletier
